﻿/****************************
* Gameboard.as 
* Gameboard class for the Build a toy game
* Represents the "game board" on which the game is played
* Responsible for destribution of items on assembly lines
* Singleton
****************************/
class Gameboard implements MyTimerListener {
	private static var instance:Gameboard = null;
	private var myRoot:MovieClip;
	private static var INIT_NUM_OF_ITEMS;
	private static var assemblyLine:Array;
	private static var itemsOnBoard:Array;
	private var t:Timer = null;
	private var maxZ:Number;
	private var pickedAssemblyLines:Array;
	private var depCenterSides:Array;
	private var ITEM_GEN_INTERVAL;
	private var allCollectedItems:Array;
	private var totalGameTime:Number;
	private var MAX_NUM_ITEMS_TO_COLLECT:Number;
	private var itemsTypeCollected:Array;
	

	/*****
				* Constructor
				*****/
	private function Gameboard(GAME_TIME) {
		//trace("an instance of class Gameboard was constructed");
		trace("gameboard: game_time ="+GAME_TIME);
		assemblyLine = new Array();
		itemsOnBoard = new Array();
		depCenterSides = new Array();
		totalGameTime = GAME_TIME;
		myRoot = Game.getRoot();
		INIT_NUM_OF_ITEMS = 10;
		maxZ = 2500;
		pickedAssemblyLines = new Array();
		ITEM_GEN_INTERVAL = 3000;
		allCollectedItems = new Array();
		MAX_NUM_ITEMS_TO_COLLECT = 3;
		itemsTypeCollected = new Array();
	}
	/************** GETTERS *****************/
	/*****
			*Getter for instance
			*Returns Gameboard instance
		*****/
	public static function getInstance(GAME_TIME):Gameboard {
		if (instance == null) {
			instance = new Gameboard(GAME_TIME);
		}
		return instance;
	}
	/************** END OF GETTERS *****************/
	/*****
				* Starts timer for items to generate
				*****/
	public function startTimer() {
		/*if (this.t != null) {
									//TODO: change timer, so it loads only once, and then resets each time we need it
									//kill old timer
									this.t.killTime();
								}*/
		this.t = new Timer(this, -1, ITEM_GEN_INTERVAL);
		// setting Gameboard as timer's listener
		this.t.startTime();
	}
	/*****
		* Adds AssemblyObjs to an array - assemblyLine
		*****/
	public function createAssemblyLine(assemblyPart:Assembly) {
		assemblyLine.push(assemblyPart);
	}
	/*****
		* Adds ItemsObjs to an array - itemsOnBoard
		*****/
	public function addItemToBoard(itemObj:Item) {
		itemsOnBoard.push(itemObj);
	}
	/*****
		* Shows game time
		*****/
	public function drawTimeLeft(timeLeft:Number) {
		//trace("total game time ="+totalGameTime);
		var timePercent = Math.round((totalGameTime-timeLeft)/totalGameTime*100);
		//trace("time percent ="+timePercent);
		var degreesPercent = Math.round(timePercent/100*360);
		//trace("degrees percent = "+degreesPercent);
		myRoot.depositCenter.clockPie.gotoAndPlay(degreesPercent/2);
		myRoot.depositCenter.gameTime.text = timeLeft;
	}
	/*****
		* Initiates the deposit of items center - for all the players
		*****/
	public function initDepositCenter() {

			depCenterSides.push(myRoot.depositCenter.depToyPart_0);
				depCenterSides.push(myRoot.depositCenter.depToyPart_1);
				depCenterSides.push(myRoot.depositCenter.depToyPart_2);
				depCenterSides.push(myRoot.depositCenter.depToyPart_3);
		
			/*	for (var i=0;i<4;i++){
					depCenterSides[i].depToyPart1.gotoAndPlay("type_"+i+"_1");
					depCenterSides[i].depToyPart2.gotoAndPlay("type_"+i+"_1");
					depCenterSides[i].depToyPart3.gotoAndPlay("type_"+i+"_1");
					depCenterSides[i]._visible = false;
				}
				*/
				for (var i=0;i<=depCenterSides.length;i++){
					depCenterSides[i].gotoAndStop(MAX_NUM_ITEMS_TO_COLLECT);
					
					depCenterSides[i].toySpot1.spot.gotoAndStop("spot_"+i);
					depCenterSides[i].toySpot2.spot.gotoAndStop("spot_"+i);
					depCenterSides[i].toySpot3.spot.gotoAndStop("spot_"+i);
					
				}
				
				
	}
	

	/*****
		* When player deposits an item, tells the "deposit center" to show the newly collected item
		*****/
	public function playerDepositedItem(playerO:Player, toyPart:Item) {
		
			//trace ("function playerDepositedItem");
			var depToyPart = toyPart;
			var	curPlayerNum = playerO.getPlayerNum();
			//trace ("cur player num =" + curPlayerNum);
				/*
				for (var i=0;i<4;i++){
					if (i == curPlayerNum){
					//depCenterSides[i]._visible = true;
					//depCenterSides[i].gotoAndStop(playerO.getNumCollectedItems());
					}
				}
				*/
				for (var i=0;i<=depCenterSides.length;i++){
					if(i == curPlayerNum){
					var numColItems = playerO.getNumCollectedItems();
					switch (numColItems){
					case 1:
						depCenterSides[i].toySpot1.gotoAndStop("toyPartSpot");
						depCenterSides[i].toySpot1.depToyPart.gotoAndStop("type_"+curPlayerNum+"_1");
						break;
					case 2:
						depCenterSides[i].toySpot2.gotoAndStop("toyPartSpot");
						depCenterSides[i].toySpot2.depToyPart.gotoAndStop("type_"+curPlayerNum+"_1");
						break;
					case 3:
						depCenterSides[i].toySpot3.gotoAndStop("toyPartSpot");
						depCenterSides[i].toySpot3.depToyPart.gotoAndStop("type_"+curPlayerNum+"_1");
						break;
					default:
					trace("error - wrong number of collected items");
					
					}
					}
				}
			depToyPart.toyDeposited();
			
			allCollectedItems.push(depToyPart);
			
	}
	
	public function addToItemsTypeCollected(iType:Number){
		itemsTypeCollected.push(iType);
		
		}
	
	/*****
			* Initilisation of falling items
			* Adding the Item objects
			* Attaching Movie Clip to Items
							*****/
	public function initItem() {
		//		INIT_NUM_OF_ITEMS
		var maxNumOfItems = 4;
		// find how many items per step should be loaded in this step
		var itemsToLoad = Math.round(Math.random()*maxNumOfItems);
		var mcZ:Number;
		pickedAssemblyLines = new Array();
		// for each item:
		for (var i = 0; i<itemsToLoad; i++) {
			mcZ = ++maxZ;
			//trace("DEPTH mcZ ="+mcZ);
			// depth
			var o = myRoot.attachMovie("item", "item_"+mcZ, mcZ);
			//	trace("o "+o);
			var itemObj = Item(o);
			addItemToBoard(itemObj);
			//select assembly line
			var myAssemblyNum = getFreeAssemblyLine();
			//	trace("item num "+i+" on assembly line "+myAssemblyNum);
			trace("itemsTypeCollected Array: "+itemsTypeCollected);
			var itemFound=false;
			var itemType;
			while (itemFound == false) {
				itemFound = true;
				itemType = Math.round(Math.random()*3);
				trace ("IT:"+itemType);
				for (var jj = 0;jj<itemsTypeCollected.length;jj++){
					trace("jj:"+jj);
					if (itemType == itemsTypeCollected[jj]){
						trace("itemType = "+itemType+" itemsTypeCollecte[jj]= "+itemsTypeCollected[jj]);
						itemFound = false;
						}				
					}
			}
			
			var itemRotation = assemblyLine[myAssemblyNum].getRotation();
			var assemblyStartPoint = assemblyLine[myAssemblyNum].getPoint();
			//TODO: change for all assemblyLines
			var point:Object = {x:assemblyStartPoint._width/2, y:assemblyStartPoint._height/2};
			assemblyStartPoint.localToGlobal(point);
			//	trace("LOCAL TO GLOBAL RESULT:"+point.x+","+point.y);
			itemObj.init(this, point.x, point.y, itemRotation, itemType, assemblyLine[myAssemblyNum]);
			assemblyLine[myAssemblyNum].addToItemsOnAssembly(itemObj);
		}
	}
	/*****
			*Responsible for finding an available assembly line to drop a new item on to.
			*Returns assemblyNum
		*****/
	public function getFreeAssemblyLine() {
		var assemblyNum:Number;
		var j = 0;
		var newNumFound = false;
		while (!newNumFound) {
			assemblyNum = Math.round(Math.random()*3);
			newNumFound = true;
			for (j=0; j<pickedAssemblyLines.length; j++) {
				if (pickedAssemblyLines[j] == assemblyNum) {
					newNumFound = false;
				}
			}
		}
		pickedAssemblyLines.push(assemblyNum);
		return assemblyNum;
	}
	/*****
		* Checks if there is a toy part at the end of an assembly line that's waiting for pick up
		*****/
	public function checkForToyPart(playerO:Player) {
		//Ask Assembly it there is "waiting for pick-up" toy part
		var waitingToyPart = assemblyLine[playerO.getPlayerNum()].getWaitingObj();
		//trace("waiting toy part "+waitingToyPart);
		// check if it's false or not
		return waitingToyPart;
	}
	/*****
		* Checks if there is an item "waiting to show" and returns it.  
		* When player takes an item, it disappears from its old assembly line, and
		* gets assigned to its new assembly line, it doesn't show up until the player actually 
		* "touches" the new assembly line
		*****/
	public function checkForWaitingToShowPart(playerO:Player) {
		var oldPlayerNum = playerO.getPlayerNum();
		if (oldPlayerNum != 0) {
			var newPlayerNum = playerO.getPlayerNum()-1;
		} else {
			var newPlayerNum = 3;
		}
		var itemToShow = assemblyLine[newPlayerNum].getWaitingToShowItem();
		return itemToShow;
	}
	/*****
		* Tells item that it was taken
		*****/
	public function playerTookItem(playerO:Player, toyPart:Item) {
		var takenToy = toyPart;
		var curPlayer = playerO;
		var itmAssembly = takenToy.getMyAssembly();
		itmAssembly.deleteFromItemsOnAssembly(takenToy);
		takenToy.toyTaken(playerO);
		
	}
	/*****
		* Assignes a taken item a new assembly line
		*****/
	public function itemAssignedNewAssembly(assemblyN:Number) {
		var oldAssemblyNum = assemblyN;
		if (oldAssemblyNum != 0) {
			var newAssembly = assemblyLine[oldAssemblyNum-1];
		} else {
			var newAssembly = assemblyLine[3];
		}
		return newAssembly;
	}
	/*****
		* Tells item that was waiting to show - to show
		*****/
	public function playerPassedItem(playerO:Player, tPart:Item) {
		var itemToShow = tPart;
		//start moving an item on the new assembly line
		//		trace ("item to show::: "+itemToShow);
		if (itemToShow != null) {
			itemToShow.itemPassed();
			var itmAssembly = itemToShow.getMyAssembly();
			itmAssembly.addToItemsOnAssembly(itemToShow);
		}
	}
	/*****
		* Removes item object from the itemsOnBoard array
		*****/
	public function removeItemFromBoard(itm:Item) {
		var itemToRemove = itm;
		for (var i = 0; i<itemsOnBoard.length; i++) {
			if (itemsOnBoard[i] == itemToRemove) {
				//trace("Removed from array Item: "+itemToRemove);
				itemsOnBoard[i] = null;
			}
		}
	}
	/*****
		* Kills the timer
		* Sends the player to a new frame - game over
		* Displays results
		*****/
	public function gameOver(winPlayer:Number) {
		//kill items generation timer
		//kill timer
		this.t.killTime();
		//show bg
		//var gameOver = myRoot.attachMovie("gameOver", "gameOver_mc", 3000);
		// trace("gameOver mc= "+gameOver);
		//gameOver._x = 160;
		//gameOver._y = 120;
		//var finalToy = myRoot.attachMovie("endToy", "endToy_mc", 3010);
		//myRoot.depositCenter.finalToy._visible = true;
		//finalToy._x = 160;
		//finalToy._y = 120;
		var numCollectedItems = allCollectedItems.length+1;
		if (numCollectedItems>=10) {
			//gradeA
		} else if (numCollectedItems>=8 && numCollectedItems<10) {
			myRoot.depositCenter.finalToy.gotoAndPlay("Agrade");
			//gradeB
		} else if (numCollectedItems>=6 && numCollectedItems<8) {
			myRoot.depositCenter.finalToy.gotoAndPlay("Bgrade");
			//gradeC
		} else if (numCollectedItems>=4 && numCollectedItems<6) {
			myRoot.depositCenter.finalToy.gotoAndPlay("Cgrade");
			//gradeD
		} else if (numCollectedItems<4) {
			myRoot.depositCenter.finalToy.gotoAndPlay("Dgrade");
			//gradeF
			myRoot.depositCenter.finalToy.gotoAndPlay("Fgrade");
		}
		//finalToy._visible = true;
		//gameOver.player.text = winPlayer;
		//move to "end game" frame
		myRoot.gotoAndPlay("endGame");
		myRoot.depositCenter.gotoAndPlay("endGame");
	}
	
	public function toString():String {
		return "Object Class:Gameboard";
	}
	/******* MyTimerListener Interface implementation *******/
	/*****
							*
					*****/
	public function timerTick(t:Timer):Void {
		//trace ();
		initItem();
	}
	/*****
							* when timeOver, gets new main card
					*****/
	public function timeOver(t:Timer):Void {
		//trace("period is over");
		//		initItem();
	}
}
