﻿/****************************
* Item.as 
* extends the MovieClip
* Responsible for items' representation on board
* Tracks the items' status.
* Creates a timer of how long the item is held at the end of each assembly line before it falls
****************************/
class Item extends MovieClip implements MyTimerListener {
	private var myGameboard:Gameboard;
	private var itemNum:Number;
	private var itemX:Number;
	private var itemY:Number;
	private var myRoot:MovieClip;
	private var gameboardInstance;
	private var myMc:MovieClip;
	private var myRotation:Number;
	private var myType:Number;
	private var toyPart:MovieClip;
	//nested MC
	private var myStatus:Number;
	//
	private var myAssembly:Assembly;
	private var t:Timer = null;
	private var END_PAUSE:Number;
	// 0 - moving on assembly line
	// 1 - waiting for pick up
	// 2 - in player's hands
	//3 - deposited
	private var isStopped:Boolean;
	/*****
					* Constructor
		*****/
	function Item() {
		gameboardInstance = Gameboard.getInstance();
		isStopped = false;
	}
	public function onEnterFrame():Void {
		if (myStatus != 0){
			return;
			}
		if (this.isStopped) {
			// Stopped - check for the "green light to go"
			if (myAssembly.getStopper() == 9999) {
				// Ok to go
				this.isStopped = false;
			} else {
				// "Red light" - stay
				this.gotoAndPlay(this._currentframe-1);
			}
		} else {
			// Not stopped - check whether need to stop
			if (myAssembly.getStopper() <= this._currentframe) {
				// Become stopped
				this.isStopped = true;
				// Change the stopframe so the next part will stop behind me
				myAssembly.setStopper(this._currentframe-7);
				// ... and wait...
				this.gotoAndPlay(this._currentframe-1);
			}
		}
	}
	/************** GETTERS *****************/
	/*****
			*Getter for item's type
			*Returns myType
		*****/
	public function getMyType() {
		return myType;
	}
	public function getMyAssembly() {
		return myAssembly;
	}
	public function getIsStopped(){
		return isStopped;
		}
	/************** END OF GETTERS *****************/
	/*****
		* Initilisation of player's items
		* Adding the Item object
		* Attaching Movie Clip to Item
		*****/
	public function init(gameboard:Gameboard, x:Number, y:Number, r:Number, type:Number, assemblyO:Assembly):Void {
		END_PAUSE = 1000;
		//END_PAUSE = 5000;
		myRoot = Game.getRoot();
		myGameboard = gameboard;
		myRotation = r;
		myType = type;
		myAssembly = assemblyO;
		itemX = x;
		itemY = y;
		this._x = itemX;
		this._y = itemY;
		this._rotation = myRotation;
		toyPart.gotoAndPlay("type_"+myType+"_1");
		myStatus = 0;
	}
	/*****
		*Starts timer for items 
		*This timer is the "pause" each item has at the end of the assembly line.
		*After the "pause" the item falls.
				*****/
	public function startTimer() {
		/*if (this.t != null) {
				//kill old timer
					this.t.killTime();
					}*/
		this.t = new Timer(this, -1, END_PAUSE);
		// setting Gameboard as timer's listener
		this.t.startTime();
	}
	/*****
		* Kills timer
		*****/
	public function endTimer() {
		//kill timer
		//trace("KILL TIMER for item:"+this);
		this.t.killTime();
	}
	/*****
		* Registers that the item has reached the end of assembly line
		* Starts item pause timer
		*****/
	public function endOfAssemblyLine() {
		//	trace("End of Assembly Line");
		myStatus = 1;
		myAssembly.addToWaitingForPickUp(this);
		startTimer();
		myAssembly.setStopper(this._currentframe-7);
		//myAssembly.setStopper(myAssembly.getStopper()+50);
		// start timer for waiting for pick up
	}
	/*****
		* Registers that player took item 
		*****/
	public function toyTaken(playerO:Player) {
		var myPlayer = playerO;
		gotoAndPlay("taken");
		myStatus = 2;
		/*if(myAssembly.getStopper()>-1){
				myAssembly.setStopper(myAssembly.getStopper()-(50));
				}
				*/
		this.isStopped = false;
		myAssembly.setStopper(9999);
		myAssembly.removeFromWaitingForPickUp(this);
		myAssembly = gameboardInstance.itemAssignedNewAssembly(myAssembly.getMyAssemblyNum());
		//		trace("my new assembly num ="+myAssembly.getMyAssemblyNum());
		myAssembly.addToWaitingToShow(this);
		this._visible = false;
		endTimer();
	}
	/*****
		* Player deposited the item
		*****/
	public function toyDeposited() {
		// add item to the array of deposited items
		// remove item from the list of items on gameboard
		gameboardInstance.removeItemFromBoard(this);
		//killMovieClip();
	}
	/*****
		* Item falls off the assembly line
		*****/
	public function breakItem() {
		gotoAndPlay("break");
	}
	public function finalBreak() {
		//trace(" -- Final breaking of item "+this);
		endTimer();
		myAssembly.setStopper(9999);
		/*if(myAssembly.getStopper()>-1){
				myAssembly.setStopper(myAssembly.getStopper()-(50));
				}*/
		//remove from "waiting for pick up"
		myAssembly.removeFromWaitingForPickUp(this);
		//		remove item from the list of items on gameboard
		gameboardInstance.removeItemFromBoard(this);
		//kill MC			
		killMovieClip();
	}
	/*****
		* player passes the item to the next assembly line
		*****/
	public function itemPassed():Void {
		//trace ("ITEM PASSED function");
		this._visible = true;
		myAssembly.removeFromWaitingToShow(this);
		myRotation = myAssembly.getRotation();
		//trace ("my rotation = "+myRotation);
		this._rotation = myRotation;
		var assemblyStartPoint = myAssembly.getPoint();
		var point:Object = {x:assemblyStartPoint._width/2, y:assemblyStartPoint._height/2};
		assemblyStartPoint.localToGlobal(point);
		this._x = point.x;
		this._y = point.y;
		this.gotoAndPlay("part_1");
		toyPart.gotoAndPlay("type_"+myType+"_1");
		myStatus = 0;
	}
	/*****
		* removes its own movieclip
		*****/
	public function killMovieClip() {
		removeMovieClip(this);
	}
	/******* MyTimerListener Interface implementation *******/
	/*****
							*
					*****/
	public function timerTick(t:Timer):Void {
		//trace ();
		breakItem();
	}
	/*****
							* when timeOver, gets new main card
					*****/
	public function timeOver(t:Timer):Void {
		//trace("period is over");
		//		initItem();
	}
}
