﻿/****************************
* Card.as 
* Card class for the cards game
* Represents player's cards that are open
* Responsible for letting the player know, if it was hit
* Resposible for reDrawing the icon
* extends the MovieClip
****************************/
class Card extends MovieClip implements MyTimerListener {
	private var myPlayer:Player;
	private var cardNum:Number;
	private var cardX:Number;
	private var cardY:Number;
	private var cardRotation:Number;
	private var myRoot:MovieClip;
	private var iconNum:Number;
	private var tableInstance;
	private var myMc:MovieClip;
	private var wrongCard:MovieClip; //nested MC
	private var myDeck:Deck;
	private var card:MovieClip;
	//nested MC
	private var t:Timer = null;
	private var xScaleSpeed:Number;
	//timer type
	private var timerType:Number;
	//no timer = 0; match = 1; cardFlip = 2
	/*****
	* Constructor
	*****/
	function Card(mc:MovieClip) {
		//trace("card constructor");
		tableInstance = Table.getInstance();
		myMc = mc;
		timerType = 0;
		
	}
	/************** GETTERS *****************/
	/*****
	*Getter for icon number
	*Returns iconNum of the card
	*****/
	public function getIconNum() {
		return iconNum;
	}
	/*****
	*Getter for card number
	*Returns cardNum of the card (0 -left one, 1 - right one)
	*****/
	public function getCardNum() {
		return cardNum;
	}
	/************** END OF GETTERS *****************/
	/*****
	* Initilisation of player's card
	* Adding the Card object
	* Attaching Movie Clip to Card
	*****/
	public function init(player:Player, myCard:Number, x:Number, y:Number, deck:Deck, r:Number):Void {
		//trace("an instance of class Card was constructed");
		myRoot = Game.getRoot();
		myPlayer = player;
		cardNum = myCard;
		myDeck = deck;
		cardX = x;
		cardY = y;
		cardRotation = r;
		this._x = cardX;
		this._y = cardY;
		this._rotation = cardRotation;
		iconNum = Math.round(Math.random()*9+1);
		card.gotoAndPlay("i_"+iconNum);
		tableInstance.addCardToTable(this);
	}
	/*****
	* Redraw the card's icon (randomly)
	*****/
	public function reDraw() {
		//		trace("function reDraw");
		iconNum = Math.round(Math.random()*9+1);
		card.gotoAndPlay("i_"+iconNum);
	}
	/*****
	* Detect if the card was pressed
	* Let player know if the card was pressed
	*****/
	public function onPress():Void {
		myPlayer.cardPressed(this);
	}
	/*****
	* Tells MC to play "spin" animation
	* Starts timer, as to when to  move matched card
	*****/
	
	public function matchAnim() {
		this.gotoAndPlay("spin");
		if (this.t != null) {
			//kill old timer
			this.t.killTime();
		}
		// start timer
		timerType = 1;
		this.t = new Timer(this, 1000, 1);
		// setting Card as timer's listener
		this.t.startTime();
	}
	
	public function wrongCardAnim(){
		this.wrongCard.gotoAndPlay("startAnim");
		}
	/*****
	* Moves the card object across the table to the player's score
	* Scales the card object down to 0
	* After scale = 0, makes the card object hidden, and puts it in its right position
	*****/
	public function moveCard() {
		switch (cardRotation) {
		case 0 :
			this._x = this._x-2;
			this._y = this._y+.3;
			break;
		case -90 :
			this._x = this._x+.3;
			this._y = this._y+2;
			break;
		case 180 :
			this._x = this._x+2;
			this._y = this._y-.3;
			break;
		case 90 :
			this._x = this._x-.3;
			this._y = this._y-2;
			break;
		default:
			trace("error - card was given wrong rotation");
		}
		if (this._xscale>0) {
			this._xscale = this._xscale-1;
			this._yscale = this._yscale-1;
		} else {
			//stop animation
			this.t.killTime();
			this._x = cardX;
			this._y = cardY;
			this._xscale = 100;
			this._yscale = 100;
			gotoAndPlay("start");
			card.gotoAndPlay("bg");
			card._visible = false;
			card.icon.gotoAndPlay("i_"+myPlayer.getPlayerNum());
			trace("hit card num was"+cardNum);
			//Tell player that it's time for a new card from the deck
			myPlayer.matchedPlayersCardIsGone(this);
		}
		
	}
	/*****
	* Flips card from BG to new card
	*****/
	public function flipCard() {
		card._visible = true;
		gotoAndPlay("start");
		//trace("slide done");
		timerType = 2;
		xScaleSpeed = -10;
		this.t = new Timer(this, -1, 25);
		// setting Table as timer's listener
		this.t.startTime();
	}
	/*****
	* Removed MC
	*****/
	public function killMovieClip() {
		removeMovieClip(this);
	}
	/*****
	* Kills timer
	* sets timerType to 0
	*****/
	public function killTimer() {
		this.t.killTime();
		timerType = 0;
	}
	/******* MyTimerListener Interface implementation *******/
	/*****
					
	*****/
	public function timerTick(t:Timer):Void {
		//trace ();
		if (timerType == 1) {
			this.moveCard();
		}
		if (timerType == 2) {
			//trace("Animation degree = 180");
			this._xscale += this.xScaleSpeed;
			if (this._xscale<=5 && this.xScaleSpeed<0) {
				this.xScaleSpeed = +10;
				reDraw();
			}
			if (this._xscale>=95 && this.xScaleSpeed>0) {
				this._xscale = 100;
				t.killTime();
				this.tableInstance.setNewCardsReady(true);
				
			}
		}
	}
	/*****
									* when timeOver, gets new main card
							*****/
	public function timeOver(t:Timer):Void {
		//trace("period is over");
	}
}
