﻿/****************************
* Player.as 
* Player class for the cards game
* Represents the player
* Responsible for assinging cards to each player
* Resposible for letting Table know player's actions
* Calculates Player's score
* Keeps count of player's deck
****************************/
class Player {
	private var score:Number;
	private var initCardsInHands:Number;
	private var cardsLeft:Number;
	private var CARDS_ON_TABLE;
	private var ACardId:Number;
	private var BCardId:Number;
	private var tableInstance;
	private var myRoot:MovieClip;
	private var playerNum:Number;
	private var ACard:Card;
	private var BCard:Card;
	// position of players' cards - starting from the 1st one counter-clockwise
	private var cardXs:Array = [105, 155, 285, 285, 212, 162, 35, 35];
	private var cardYs:Array = [205, 205, 177, 127, 35, 35, 70, 120];
	private var rotations:Array = [0, 0, -90, -90, 180, 180, 90, 90];
	private var deckInstance:Deck;
	private var deckXs:Array = [205, 285, 112, 35];
	private var deckYs:Array = [205, 77, 35, 170];
	private var deckOffsetXs:Array = [1, 0, -1, 0];
	private var deckOffsetYs:Array = [0, -1, 0, 1];
	private var decPosX:Number;
	private var decPosY:Number;
	private var deckOffsetX:Number;
	private var deckOffsetY:Number;
	/*****
							* Constructor
					*****/
	public function Player(initCards:Number, player:Number) {
		//trace("an instance of class Player was constructed");
		// initCardsInHands constitutes the number of "closed cards" in the deck
		initCardsInHands = initCards;
		CARDS_ON_TABLE = 2;
		// cards left accounts for initCardsInHands plus the cards that are on the table.
		cardsLeft = initCardsInHands+CARDS_ON_TABLE;
		tableInstance = Table.getInstance();
		myRoot = Game.getRoot();
		playerNum = player;
		score = 0;
		Game.getDpem().addDPadListener(this, playerNum-1);
	}
	/************** GETTERS *****************/
	/*****
			*Getter for playerNum
			*Returns Player's number
			*****/
	public function getPlayerNum():Number {
		return playerNum;
	}
	/*****
					/************** END OF GETTERS *****************/
	/*****
			* Tells player's deck to init
			* Initialises the cards for the player
			* Each player has 2 cards (ACard and BCard (0 and 1))
			* Attaches MovieClip to each player's card
					*****/
	public function initCards() {
		// calculate deck position
		decPosX = deckXs[(playerNum-1)];
		deckOffsetX = deckOffsetXs[(playerNum-1)];
		decPosY = deckYs[(playerNum-1)];
		deckOffsetY = deckOffsetYs[(playerNum-1)];
		//init decks
		initDecks();
		//init cards
		var i:Number = 0;
		var mcZ:Number;
		var mcName:String;
		var crd:Card;
		var cardRotation:Number;
		// creating ACard
		mcZ = playerNum*10;
		// depth
		mcName = "card_mc_"+mcZ;
		var o = myRoot.attachMovie("card_full", mcName, mcZ);
		ACard = Card(o);
		ACard.init(this, 0, cardXs[(playerNum-1)*2], cardYs[(playerNum-1)*2], deckInstance, rotations[(playerNum-1)*2]);
		// creating BCard
		mcZ = playerNum*10+1;
		// depth		
		mcName = "card_mc_"+mcZ;
		var o = myRoot.attachMovie("card_full", mcName, mcZ);
		BCard = Card(o);
		BCard.init(this, 1, cardXs[(playerNum-1)*2+1], cardYs[(playerNum-1)*2+1], deckInstance, rotations[(playerNum-1)*2]);
	}
	/*****
				* Initialises the decks for the player
				* Creates Deck obj for the player
			*****/
	public function initDecks() {
		//trace ("function initDecks");
		deckInstance = new Deck(decPosX, decPosY, deckOffsetX, deckOffsetY);
		deckInstance.init(initCardsInHands, this);
	}
	/*****
			* Asks card to reDraw its icon
			*****/
	public function getCards(cardO:Card) {
		cardO.reDraw();
	}
	/*****
			* Asks Table to check if the pressed card and the main card match
			* IF THERE IS A MATCH:
			* Tells card to play "match anim"
			* Adds score
			* Subtracts a card from cardsLeft
			* Checks if player still has a deck (not opened cards)
			* Tells Table to redraw the main card					
			*****/
			
	public function onControlPressed(code) {
		trace ("function onControlPressed");
		switch (code) {
		case "A" :
		trace ("Player pressed A");
		cardPressed(ACard);

			break;
		case "B" :
		trace ("Player pressed B");
		cardPressed(BCard);
			break;
		}
	}
			
	public function cardPressed(cardO:Card) {
		//tells Table that you've been hit
		var cardMatch:String = tableInstance.checkCard(cardO.getIconNum());
		trace("on cardPressed cardMatch ="+cardMatch);
		// match/nomatch/taken/busy
		switch (cardMatch) {
		case "match" :
			//play "match animation"
			cardO.matchAnim();
			//add points
			score++;
			//trace("my score = "+score);
			var scorePlayer = myRoot["score_"+playerNum];
			//trace("score player = "+scorePlayer);
			scorePlayer.text = score;
			//check if there are any cards in deck
			//subtract 1 card from deck
	/*		var deck:Boolean;
			cardsLeft--;
			deck = deckInstance.subtractCard();
			//returns true/false if false kill card on table
				if (deck == false) {
							//kill the pressed card
							trace("Deck == false. Cards Match. Player has no more cards in deck");
							tableInstance.deleteCardFromTable(cardO);
							cardO.killMovieClip();
							if (cardsLeft == 0) {
								trace("cardsLeft ="+cardsLeft);
								tableInstance.gameOver(playerNum);
							} else {
								trace("cardsLeft ="+cardsLeft);
								//change main card
								tableInstance.getMainCard();
							}
						} else {
							trace("Cards Match. Player still has a deck");
							//change main card
							tableInstance.getMainCard();
						}*/
			break;
		case "aftermatch" :
			trace("after match");
			break;
		case "nomatch" :
			trace("cards don't match");
			cardO.wrongCardAnim();
			break;
		case "taken" :
			trace("card is taken");
			break;
		case "busy" :
			trace("card is busy");
			break;
		default :
			trace("error - there is no status like this between pressed card and main card");
		}
	}
	/*****
			* Tells deck to start sliding a new card
			*****/
	public function matchedPlayersCardIsGone(playersCard:Card) {
		

		var deck:Boolean;
		cardsLeft--;
		deck = deckInstance.subtractCard(playersCard);
		if (deck == false) {
							//kill the pressed card
							trace("Deck == false. Cards Match. Player has no more cards in deck");
							tableInstance.deleteCardFromTable(playersCard);
							playersCard.killMovieClip();
							if (cardsLeft == 0) {
								trace("cardsLeft ="+cardsLeft);
								tableInstance.gameOver(playerNum);
							} else {
								trace("cardsLeft ="+cardsLeft);
								deckInstance.slideCard(playersCard);
								//change main card
								tableInstance.getMainCard();
							}
						} else {
							trace("Cards Match. Player still has a deck");
							//deckInstance.slideCard(playersCard);
							//change main card
							tableInstance.getMainCard();
						}
		//deckInstance.slideCard(playersCard);
	}
	/*****
			* Tells card that was just matched to start flipping to a new card
			*****/
	public function newCardSlideDone(playersCard:Card) {
		playersCard.flipCard();
	}
}
