﻿/****************************
* Table.as 
* Table class for the cards game
* Represents the "table" on which the game is played
* Responsible for coordinating players' actions and cards
* Singleton
****************************/
class Table implements MyTimerListener {
	private static var instance:Table = null;
	private var curCardId:Number;
	private var myRoot:MovieClip;
	//IconNum of the Main Card
	private var cardTimer:Number;
	private var tStatus:Number;
	/* status can be:
								// 0 = open
								// 1 = taken
								// 2 = busy
								*/
	private var mcName:String;
	private var t:Timer = null;
	private var mainCard:MainCard;
	private var cardsOnDesk:Array;
	public var matchAnimFlag:Boolean;
	private var newCardsReady:Boolean;
	// flag - is the main card "match" anim playing
	/*****
				* Constructor
				*****/
	private function Table() {
		//trace("an instance of class Table was constructed");
		tStatus = 0;
		cardsOnDesk = new Array();
		newCardsReady = false;
	}
	/************** GETTERS *****************/
	/*****
							*Getter for instance
							*Returns Table instance
							*****/
	public static function getInstance():Table {
		if (instance == null) {
			instance = new Table();
		}
		return instance;
	}
	/*****
							*Getter for curCardId
							*Returns curCardId - IconNum of the Main Card
							*****/
	public function getCurCardId():Number {
		return curCardId;
	}
	/************** END OF GETTERS *****************/
	/*****
							* Initilisation of Main Card
							* Adding the MainCard object
							* Attaching Movie Clip to MainCard
							*****/
	public function initMainCard() {
		prepareNewCard();
		mcName = "card_main";
		//		trace(" Creating main card");
		myRoot = Game.getRoot();
		var o = myRoot.attachMovie("mainCard", mcName, 9);
		mainCard = MainCard(o);
		mainCard.init(curCardId, 160, 120);
		matchAnimFlag = false;
	}
	/*****
							* Prepares Main Card
							* Gets number of cards on table
							* Randomly takes one of the cards on table and gets its iconNum, and changes curCardId to it
							* Kills old timer, Starts new timer - the time the main card is shown on table for
					*****/
	public function prepareNewCard() {
		// get number of cards in cardsOnDesk
		var arrLength:Number = cardsOnDesk.length-1;
		var arrCell:Number = Math.round(Math.random()*arrLength);
		var newCard:Card;
		// trace("Index="+arrCell);
		newCard = Card(cardsOnDesk[arrCell]);
		// trace("main card id ="+newCard);
		curCardId = newCard.getIconNum();
		if (this.t != null) {
			//kill old timer
			this.t.killTime();
		}
		// start timer
		this.t = new Timer(this, 3000, 1000);
		// setting Table as timer's listener
		this.t.startTime();
	}
	/*****
							* Adds player's card object to the array of all cards on table - cardsOnDesk
					*****/
	public function addCardToTable(cardO:Card) {
		cardsOnDesk.push(cardO);
		//trace("cards on desk ="+cardsOnDesk);
	}
	public function deleteCardFromTable(cardO:Card) {
		var i = 0;
		var cardToDelete:Number = -1;
		while (i<cardsOnDesk.length) {
			if (cardsOnDesk[i] == cardO) {
				cardToDelete = i;
			}
			i++;
		}
		if (cardToDelete == -1) {
			//error
			trace("no such card on table");
			return;
		}
		while (cardToDelete+1<cardsOnDesk.length) {
			cardsOnDesk[cardToDelete] = cardsOnDesk[cardToDelete+1];
			cardToDelete++;
		}
		cardsOnDesk.pop();
	}
	/*****
							* Gets new Main card - changes the icon of main card
					*****/
	public function getMainCard() {
		trace("function getMainCard - status = "+tStatus);
		trace("newCardsReady "+newCardsReady);
		if (tStatus == 0) {
			prepareNewCard();
			mainCard.reDraw(curCardId);

			//TODO: change the place where this happens
		} else if (tStatus == 1) {
			// match animation is playing
			if (matchAnimFlag == false) {
				mainCard.matchAnimMain();
				matchAnimFlag = true;
			}
			prepareNewCard();
			// prepare card, but we don't use it, we'll just use the timer
		} else if (tStatus == 5 && newCardsReady == false) {
			// after being taken/ anim of player's cards is not done yet
			prepareNewCard();
		}		else if (tStatus == 5 && newCardsReady == true) {
			// after being taken, now free to be changed to open
			prepareNewCard();
			mainCard.reDrawInit(curCardId);
			tStatus = 0;
			newCardsReady = false;
		}
	}
	public function setStatus(s:Number) {
		tStatus = s;
		trace("Table: status = "+tStatus);
	}
	
	public function setNewCardsReady(ncr:Boolean){
		newCardsReady = ncr;
		}
	/*****
							* Checks the card that was pressed with the current Main card 
							* Returns the result (match/nomatch/taken/busy)
					*****/
	public function checkCard(pressedCardId:Number) {
		switch (tStatus) {
		case 0 :
			//this is a match!
			//trace("open");
			if (pressedCardId != curCardId) {
				//cards don't match
				return "nomatch";
			} else {
				//cards match
				tStatus = 1;
				//taken
				return "match";
			}
			break;
		case 1 :
			//trace("taken");
			return "taken";
			break;
		case 2 :
			//trace("Main Card is Busy");
			return "busy";
			break;
		case 5:
		//right after playing match anim
			return "aftermatch";
			break;
		default :
			trace("error - no such status");
		}
	}
	public function gameOver(winPlayer:Number) {
		//kill timer
		killTimer();
		//////////////
		myRoot.gotoAndPlay("endGame");
	//	var gameOver = myRoot.attachMovie("gameOver", "gameOver_mc", 2000);
	//	gameOver._x = 160;
	//	gameOver._y = 120;
	//	gameOver.player.text = winPlayer;
	myRoot.card_main._visible = false;
	myRoot["player_"+winPlayer].gotoAndPlay("won");
	
	}
	public function killTimer() {
		this.t.killTime();
	}
	/******* MyTimerListener Interface implementation *******/
	/*****
							*
					*****/
	public function timerTick(t:Timer):Void {
		//trace ();
	}
	/*****
							* when timeOver, gets new main card
					*****/
	public function timeOver(t:Timer):Void {
		//trace("period is over");
		getMainCard();
	}
}
