﻿/**
* Class Ball - represents the ball movieclip
**/

class Ball extends MovieClip implements AnimationTickerListener {
	private static var SHOOT_VEL = 3; // Shooting velocity (px/tick)
	private static var MAX_VEL = 6; // maximal possible velocity (px/tick)
	private static var MIN_VEL = 2; // minimal possible velocity (px/tick)
	private static var MAX_AXIS_VEL = 5.8; // maximal possible velocity along an axis (px/tick)
	private static var MIN_AXIS_VEL = .2; // minimal possible velocity  along an axis (px/tick)
	private static var SPEED_COEFF = .2; //coefficient to adjust the speed of the ball when it hitsa a moving object
	private static var SPEED_DECAY = .0015; // Coefficient showing how fast the speed
	                    				// will try to return to "normal"
	private static var SPEED_INCREMENT = 1; //slowly increase minimal and "normal" speed                    				
	private var velX:Number; // X axis velocity
	private var velY:Number; // Y axis velocity
	private var r:MovieClip; // parent movieclip
	private var normalSpeed:Number;
	
	private var testBall:MovieClip;
	
	//----- collision detection data
	
	private var nextX:Number;
	private var nextY:Number;
	
	
	public function getVelX() {
		return velX;
	}
	public function getVelY() {
		return velY;
	}
	
	public function getTestBall():MovieClip {
		return this.testBall;
	}
	
	public function init() {
		//this._visible = false;
		
		testBall = this.duplicateMovieClip("tstBall", this.getNextHighestDepth());
		testBall._visible = false;
		
	}
		
	public function getNextX(){
		return nextX;		
	}

	public function getNextY(){
		return nextY;		
	}
    

	
	/**
	* Positions the ball at the game field center
	**/

	public  function center(): Void{
		this._x = 160;
		this._y = 120;
	}

	/**
	* Starts the ball movement from the center into the given direction
	* Arguments:
	* degree - direction (in degrees, 0-360)
	**/

	public function shoot(degree:Number): Void{
		r= CastleGame.getRoot();
		var rads = degree * Math.PI/180;
		velX = Ball.SHOOT_VEL*Math.cos(rads);
		velY = Ball.SHOOT_VEL*Math.sin(rads);
		normalSpeed = SHOOT_VEL;
		startMovement();
	}

	/**
	* Starts the ball's animation timer
	**/
	
	public function startMovement():Void{
		AnimationTicker.getInstance().addListener(this);
	}

	/**
	* Stops the animation timer
	**/

	public function stopMovement():Void{
		AnimationTicker.getInstance().removeListener(this);
		
	}
	
	public function speedUp():Void{
		r.speedUp.gotoAndPlay("show");
		MIN_VEL+=SPEED_INCREMENT;
		MAX_VEL+=SPEED_INCREMENT;
		normalSpeed+=SPEED_INCREMENT;
		var velMod = Math.sqrt(this.velX*this.velX + this.velY*this.velY);
		var sinA = velY/velMod;
		var cosA = velX/velMod;
		this.velY = normalSpeed*sinA; 
		this.velX = normalSpeed*cosA; 
		trace("Speeding up!");

	}
	/**
	* Calculates new speed of the ball based on the current speed and the speed of the 
	* object that hit the ball
	* Arguments:
	* oldSpeed - current speed of the ball
	* hitSpeed -  the speed of the object that hit the ball
	* Returns: new speed
	**/

	private function adjustSpeed(oldSpeed:Number, hitSpeed:Number):Number{
		var newSpeedMod = Math.abs(oldSpeed+hitSpeed*SPEED_COEFF);
		if (newSpeedMod>MAX_AXIS_VEL) { return oldSpeed;}
		if (newSpeedMod<MIN_AXIS_VEL) { return oldSpeed;}
		return oldSpeed+hitSpeed*SPEED_COEFF;
	}

	/**
	* Changes the ball's X and Y speed if the ball was reflected upwards
	* Arguments:
	* hitVel - velocity of the object the ball is reflected from
	**/

	public function reflectUp(hitVel:Number): Void{
		trace ("reflect up");
		velX=adjustSpeed(velX, hitVel);
		velY = Math.abs(velY);
		velY *= -1;
//			this._x += this.velX;
//			this._y += this.velY;
		
//		this.nextX = this._x + vx;
//		this.nextY = this._y + vy;
	}

	/**
	* Changes the ball's X and Y speed if the ball was reflected downwards
	* Arguments:
	* hitVel - velocity of the object the ball is reflected from
	**/

	public function reflectDown(hitVel:Number){
		trace ("reflect down");
		
		velX=adjustSpeed(velX, hitVel);
		velY = Math.abs(velY);
//		this._x += this.velX;
//		this._y += this.velY;
		
//		this.nextX = this._x + vx;
//		this.nextY = this._y + vy;
	}

	/**
	* Changes the ball's X and Y speed if the ball was reflected to the left
	* Arguments:
	* hitVel - velocity of the object the ball is reflected from
	**/

	public function reflectLeft(hitVel:Number){
		trace ("reflect left");
		
		velY=adjustSpeed(velY, hitVel);
		velX = Math.abs(velX);
		velX *= -1;
//			this._x += this.velX;
//			this._y += this.velY;

		//		this.nextX = this._x + vx;
//		this.nextY = this._y + vy;
		
	}

	/**
	* Changes the ball's X and Y speed if the ball was reflected to the right
	* Arguments:
	* hitVel - velocity of the object the ball is reflected from
	**/

	public function reflectRight(hitVel:Number){
		trace ("reflect right");
		
		velY=adjustSpeed(velY, hitVel);
		velX = Math.abs(velX);
//			this._x += this.velX;
//			this._y += this.velY;
//		this.nextX = this._x + vx;
//		this.nextY = this._y + vy;
	}
	
	/**
	* Animation tick. This function is called by the animation timer repeatedly with the 
	* specified interval. It changes the coordinates of the ball and checks for the collision. 
	**/

	
	public function animTick():Void{
		
		// adjust speed
		
		var velMod = Math.sqrt(this.velX*this.velX + this.velY*this.velY);
		var sinA = velY/velMod;
		var cosA = velX/velMod;
		//
		if (velMod != normalSpeed) {
			// Adjust speed
			if (Math.abs(velMod-normalSpeed)<SPEED_DECAY) {
				velMod = normalSpeed;
			}
			else {
				// small adjustment
				velMod+=(velMod < normalSpeed)?SPEED_DECAY:-SPEED_DECAY;
			}
		}
		
		velX = velMod*cosA;
		velY = velMod*sinA;
		
		
//do {	
//		trace ("velX:"+velX+" velY:"+velY);
		this.nextX = this._x + velX;
		this.nextY = this._y + velY;
		
		this.testBall._x = this.nextX;
		this.testBall._y = this.nextY;
		var c = false;
		// Check for collisions with all players
		if (Player.testCollision(this)) {
			c=true;
//			this._x += this.velX;
//			this._y += this.velY;
//			return;
		}
		
		// Check for collisions with all walls

		if (Wall.testCollision(this)) {
			c=true;
			// collision detected
//			this._x += this.velX;
//			this._y += this.velY;
//			return;
		}

		if (SolidBlock.testCollision(this)) {
			c=true;
//			return;
		}
		
		// Check for collision with boundary
		if (testBall.hitTest(r.top_wall) ){
			c=true;
			this.reflectDown(0);
		}
		if (testBall.hitTest(r.left_wall)){
			c=true;
			this.reflectRight(0);
		}
		if (testBall.hitTest(r.bottom_wall)){
			c=true;
			this.reflectUp(0);
		}
		if (testBall.hitTest(r.right_wall)){
			c=true;
			this.reflectLeft(0);
		}
//} while (c);
//		if (!c) {
		this._x += this.velX;
		this._y += this.velY;
//		}

		updateAfterEvent();
	}
	
}