﻿/*********************************************
* the main class for the Castle defence game
**********************************************/
class CastleGame {
	private static var root:MovieClip;
	private static var dpem:DPadEmulator = null;
	private var playersAlive:Array;
	// List of players alive - when the player loses, he is deleted from the array
	private var numOfPlayersAlive:Number;
	// Number of players alive
	private var ball:Ball;
	/**
		* This function is used to rotate coordinates around the game field center, according to the player num
		* Arguments:
		* pNum - player number for whom to translate coordinates
		* c: coordinates to translate
		* Returns: new Coords object containing the translated coordinates
		**/
	public static function rotateCoords(pNum:Number, c:Coords) {
		var xCent = 160;
		var yCent = 120;
		var rot = Math.PI/2*pNum;
		var newX = xCent+(c.getX()-xCent)*Math.cos(rot)-(c.getY()-yCent)*Math.sin(rot);
		var newY = yCent+(c.getX()-xCent)*Math.sin(rot)+(c.getY()-yCent)*Math.cos(rot);
		return new Coords(newX, newY);
	}
	/**
		* Constructor
		**/
	public function CastleGame() {
		this.playersAlive = new Array();
		this.numOfPlayersAlive = 0;
		this.ball = root.cball;
	}
	public function getBall():Ball {
		return root.cball;
	}
	/**
		* Entry point. 
		* Parameters:
		* root: the parent movieclip
		**/
	public static function main(rootmc:MovieClip):Void {
		root = rootmc;
		var game:CastleGame = new CastleGame();
		game.startGame();
	}
	/**
		* Creates a new instance of DPAdEmulator if necessary and returns it
		* Returns: DPadEmulator instance
		**/
	public static function getDpem():DPadEmulator {
		if (dpem == null) {
			dpem = DPadEmulator.getInst(root);
		}
		return dpem;
	}
	/**
		* Getter for root
		* Returns: root MC
		**/
	public static function getRoot():MovieClip {
		return root;
	}
	/**
		* Starts game
		**/
	public function startGame():Void {
		trace("Starting game...");
		initGame();
	}
	/**
		* Initializes game MCs, and starts the gun
		**/
	public function initGame():Void {
		root.player1.initialize(0, 160, 118, this, root.lost_txt_0, root.char_0);
		root.player2.initialize(1, 160, 118, this, root.lost_txt_1, root.char_1);
		root.player3.initialize(2, 160, 118, this, root.lost_txt_2, root.char_2);
		root.player4.initialize(3, 160, 118, this, root.lost_txt_3, root.char_3);
		// assign walls to players
		root.wall1.init(root.player1, Wall.POS_BOTTOM);
		root.wall2.init(root.player1, Wall.POS_BOTTOM);
		root.wall3.init(root.player1, Wall.POS_BOTTOM);
		root.wall4.init(root.player1, Wall.POS_BOTTOM);
		root.wall5.init(root.player4, Wall.POS_RIGHT);
		root.wall6.init(root.player4, Wall.POS_RIGHT);
		root.wall7.init(root.player4, Wall.POS_RIGHT);
		root.wall8.init(root.player4, Wall.POS_RIGHT);
		root.wall9.init(root.player3, Wall.POS_TOP);
		root.wall10.init(root.player3, Wall.POS_TOP);
		root.wall11.init(root.player3, Wall.POS_TOP);
		root.wall12.init(root.player3, Wall.POS_TOP);
		root.wall13.init(root.player2, Wall.POS_LEFT);
		root.wall14.init(root.player2, Wall.POS_LEFT);
		root.wall15.init(root.player2, Wall.POS_LEFT);
		root.wall16.init(root.player2, Wall.POS_LEFT);
		root.cball.init();
		root.gun.init(root);
		root.gun.shoot(root.cball);
		root.lost_txt_0._visible = false;
		root.lost_txt_1._visible = false;
		root.lost_txt_2._visible = false;
		root.lost_txt_3._visible = false;
		//		root.cball.shoot(197);
	}
	/**
		* This function is called by a Player object, and records the player number in
		* the playersAlive array and counts player in numOfPlayersAlive
		* (used to avoid hardcoding the number of active players)
		* Arguments:
		* pNum - player number 
		**/
	public function registerPlayer(pNum:Number):Void {
		trace("Registering player "+pNum);
		playersAlive[pNum] = true;
		numOfPlayersAlive++;
	}
	/**
		* This function is called by a player object when all walls of this player are destroyed.
		* the function checks whether only one player is still active, and, if yes, initiate the endgame sequence
		* Arguments:
		* pNum - player number 
		**/
	public function playerLost(pNum:Number):Void {
		playersAlive[pNum] = false;
		numOfPlayersAlive--;
		trace(numOfPlayersAlive+"players left");
		root.cball.speedUp();
		if (numOfPlayersAlive == 1) {
			trace("Looking for a winner...");
			// only one player left - end game
			for (var i = 0; i<playersAlive.length; i++) {
				trace("Player "+i+" is "+playersAlive[i]);
				if (playersAlive[i]) {
					var plNum = i+1;
					root["player"+plNum].won();
					root.cball.stopMovement();
					root.gotoAndPlay("Gamover");
					root.WinnerText._visible = false;
					if (i == 0) {
						root.winnerArrow._rotation = -45;
						root.gameOverTxt._rotation = 0;
					} else if (i == 1) {
						root.winnerArrow._rotation = 45;
						root.gameOverTxt._rotation = 90;
					} else if (i == 2) {
						root.winnerArrow._rotation = 135;
						root.gameOverTxt._rotation = 180;
					} else if (i == 3) {
						root.winnerArrow._rotation = -135;
						root.gameOverTxt._rotation = -90;
					}
					//					root.WinnerText.text = "Winner is player "+i;
					trace("Game over - winner is player "+i);
				}
			}
		}
	}
}
