﻿/*
 * DPadEmulator.as
 * captures keystrokes and sends to all subscribed listener objects
 */
	
class DPadEmulator {
	private static var inst:DPadEmulator=null;			//Singleton instance
	private var root:MovieClip;						//reference to root timeline
	
	private var p0Listeners:Array = new Array();	//all listeners to d-pad 0
	private var p1Listeners:Array = new Array();	//all listeners to d-pad 1
	private var p2Listeners:Array = new Array();	//all listeners to d-pad 2
	private var p3Listeners:Array = new Array();	//all listeners to d-pad 3
	
	//keyboard controls
	private var CONTROL_CODES:Array = ["U", "R", "D", "L", "B", "A"];	//control codes sent to listeners
	private var P0_CONTROLS:Array = [82, 70, 67, 68, 86, 71];		//[ R, F, C, D, V, G ]
	private var P1_CONTROLS:Array = [50, 87, 65, 81, 83, 90];		//[ 2, W, A, Q, S, Z ]
	private var P2_CONTROLS:Array = [85, 74, 78, 72, 77, 75];		//[ U, J, N, H, M, K ]
	private var P3_CONTROLS:Array = [219, 222, 191, 186, 80, 189];	//[ [, ', /, ;, P, - ]

	
	private function DPadEmulator (_r:MovieClip) {
		root = _r;
		Key.addListener(this);
	}
	
	public static function getInst (_r:MovieClip):DPadEmulator {
		if (inst==null) { 
			inst = 	new DPadEmulator(_r);
		}
		 return inst; 
	}
	
	public function addDPadListener (lObj:Object, padNum:Number) :Void {
		trace ("registered listener for dpad "+padNum);
		var tList:Array = this["p"+padNum+"Listeners"];
		tList.push(lObj);
	}
	
	public function removeDPadListener (lObj:Object, padNum:Number) :Void {
		var tList = this["p"+padNum+"Listeners"];
		var numLs = tList.length;
		var lObjFound:Boolean = false;
		
		//remove all occurrences of lObj
		for (var i=0; i<numLs; i++) {
			if (tList[i] == lObj) {
				tList.splice(i, 1);
				lObjFound = true;
			}
		}
		
		if (!lObjFound) { trace("ERROR:: DPadEmulator.removeDPadListener: listener "+lObj+" not found"); }
	}
	
	public function onKeyDown () {
		var kNum:Number = Key.getCode();
		
		for (var i=0; i<6; i++) {
			for (var j=0; j<4; j++) {
				if (kNum == this["P"+j+"_CONTROLS"][i]) {
					broadcastCode(j, i, true);
					return;
				}
			}
		}
	}
	
	public function onKeyUp () {
		var kNum:Number = Key.getCode();
		
		for (var i=0; i<6; i++) {
			for (var j=0; j<4; j++) {
				if (kNum == this["P"+j+"_CONTROLS"][i]) {
					broadcastCode(j, i, false);
					return;
				}
			}
		}
	}
	
	private function broadcastCode (playerNum:Number, controlNum:Number, pressed:Boolean) :Void {
		var tList = this["p"+playerNum+"Listeners"];
		var numLs = tList.length;
		
		for (var i=0; i<numLs; i++) {
			if (pressed) { tList[i].onControlPressed(CONTROL_CODES[controlNum]); }
			else { tList[i].onControlReleased(CONTROL_CODES[controlNum]); }
		}
	}
	
	public function isKeyDown (pNum:Number, cCode:String) :Boolean {
		var controlNum:Number;
		var numCodes = CONTROL_CODES.length;
		for (var i=0; i<numCodes; i++) {
			if (CONTROL_CODES[i] == cCode) {
				controlNum = i;
				break;
			}
		}
		if (!controlNum) { trace("ERROR:: DPadEmulator.as: invalid control code passed to DPadEmulator.isKeyDown"); }
		
		return (Key.isDown(this["P"+pNum+"_CONTROLS"][controlNum]));
	}
}
