﻿/**
* Class Player - represents the paddle MC
**/

class Player extends MovieClip implements AnimationTickerListener {
	private static var PADDLE_SPEED =6; // The paddle movement speed
	private static var instances:Array = new Array(); // collection of player instances. Used to facilitate collision testing.
	
	private var num:Number; // Player number (0-based)
	private var relX:Number; // Relative X coordinate
	private var relY:Number;  // Relative Y coordinate
	private var velX:Number; // X speed
	private var leftLimit:Number = 90; // The limit to the left movement
	private var rightLimit:Number = 217; // The limit to the right movement
	
	private var numWalls:Number; // Number of walls the player currently has 
	private var g:CastleGame; // The game itself

	private var testPaddle:MovieClip;
	private var ballMC:Ball;
	//private var collider:CollidingFrame;

	private var isAlive:Boolean;
	private var myLostTxt:TextField;
	private var gameCharacter:MovieClip;
	/**
	* This function is called by constructor. It adds the new instance of the Player class
	* to the collection.
	* Arguments:
	* inst - instance of the player class
	**/

	private static function addInstance(inst:Player):Void {
		instances.push(inst);
	}

	/**
	* This function iterates over all registered instances and 
	* tests each for collision with the supplied Ball object.
	* Arguments:
	* obj - Ball object
	* Returns:
	* true if collision was detected; false otherwise.
	**/
	
	public static function testCollision(obj:Ball):Boolean {
		for (var w in instances) {
			if (instances[w].testColl(obj)) {
				return true;
			}
		}
			return false;
	}

	
	
	/**
	* Initializes object, positions it to the correct initial position,
	* registers it as DPad listener and starts the movement timer.
	* Arguments:
	* id - player number
	* x - x coordinate of the initial position (relative)
	* y - y coordinate of the initial position (relative)
	* game - game object
	**/
	public function initialize(id:Number, x:Number, y:Number, game:CastleGame, lostTxt:TextField, gChar:MovieClip):Void {
		
		g=game;
		Player.addInstance(this);
		g.registerPlayer(id);
		num = id;
		numWalls = 0;
		trace("I am player number "+num+". My rotation is "+this._rotation);
		this.relX = x;
		this.relY = y;
		var c:Coords = CastleGame.rotateCoords(id, new Coords(x, y));
		this._x = c.getX();
		this._y = c.getY();
		this.velX = 0;
		testPaddle = this.duplicateMovieClip("tstPad"+num, this.getNextHighestDepth());
		testPaddle._visible = true;
		CastleGame.getDpem().addDPadListener(this, num);
		ballMC = g.getBall();
		trace("test Paddle:"+testPaddle);
		isAlive = true;
		startMovement();
		myLostTxt = lostTxt;
		gameCharacter = gChar;
	}


	/**
	* Increase count of the walls the player has
	**/
	public function addWall(w:Wall){
		this.numWalls++;
	}

	/** 
	* The function is called when a wall is being destoryed.
	* Decrements the number of walls. If the number is 0 - the player 
	* had lost. Then the timer is stopped, the graphics is changed
	* and the game is notified.
	* Arguments: 
	* w - wall object
	**/
	
	public function removeWall(w:Wall){
		this.numWalls--;
		//gameCharacter.gotoAndPlay("sad");
		if (this.numWalls <= 0 ){
			//Player lost
			trace("Player "+num + " has lost ");
			lose();
		}
	}
	public function gotHit(){
		gameCharacter.gotoAndPlay("sad");
		}
	public function lose(){
		isAlive = false;
		gameCharacter.gotoAndPlay("lost");
		//this.gotoAndPlay("dead");
		this._visible = false;
		myLostTxt._visible = true;
		AnimationTicker.getInstance().removeListener(this);
		g.playerLost(num);
	}
	public function won(){
		trace("***** function won :: in Player class **********");
		gameCharacter.gotoAndPlay("won");
		myLostTxt.text = "WINNER";
		myLostTxt._visible = true;

		}
	
	public function startMovement() {
		AnimationTicker.getInstance().addListener(this);
	}
	
	public function animTick():Void{
		var oldRelX = this.relX;
		if (CastleGame.getDpem().isKeyDown(this.num,"L")) {
			 this.velX = -PADDLE_SPEED; 
		} else	if (CastleGame.getDpem().isKeyDown(this.num,"R")) {
			this.velX = PADDLE_SPEED; 
		} else {
			this.velX = 0; 
		}
			
		if (this.relX+this.velX<this.leftLimit) {
			this.relX=this.leftLimit;
		} else
		if (this.relX+this.velX>this.rightLimit) {
			this.relX=this.rightLimit;
		} else {
			this.relX += this.velX;
		}
		var c:Coords = CastleGame.rotateCoords(this.num, new Coords(this.relX, this.relY));
		this.testPaddle._x = c.getX();
		this.testPaddle._y = c.getY();
		if (testPaddle.hitTest(ballMC)){
			trace ("hit ball");
			   this.velX = 0;
			   this.relX = oldRelX;
		} else {
			this._x = c.getX();
			this._y = c.getY();
		}
		if (hitTest(ballMC)){
			this.relX = oldRelX; 
			var c:Coords = CastleGame.rotateCoords(this.num, new Coords(this.relX, this.relY));
			this._x = c.getX();
			this._y = c.getY();
		}
	}

/*	
	public function onControlPressed(code) {
	}
	public function onControlReleased(code) {
		this.velX = 0;
	}
	*/
	
	public function testColl(targ:Ball):Boolean {
		 if (!isAlive) {
			 return false;
			}
		if (!hitTest(targ.getTestBall())){
			return false;
		}
		trace ("player hit!");
		var bounds:Object = this.getBounds(CastleGame.getRoot());
		var xDir = 0;
		var yDir = 0;
		
		if (targ._x <= bounds.xMin) { xDir = -1; }
		if (targ._x >= bounds.xMax) { xDir = 1; }
		if (targ._y <= bounds.yMin) { yDir = -1; }
		if (targ._y >= bounds.yMax) { yDir = 1; } 
		
		if (yDir == 1){ 
			targ.reflectDown(velX);
			}
		if (yDir == -1){ 
			targ.reflectUp(velX);
			}
		if (xDir == 1){ 
			targ.reflectRight(velX);
			}
		if (xDir == -1){ 
			targ.reflectLeft(velX);
			}
		return true;

	}
}
