﻿/**
* Class SolidBlock
**/

class SolidBlock extends MovieClip{
	private static var instances:Array = new Array(); // collection of solid block instances. Used to facilitate collision testing.
	
	/**
	* This function is called by constructor. It adds the new instance of the Player class
	* to the collection.
	* Arguments:
	* inst - instance of the player class
	**/

	private static function addInstance(inst:SolidBlock):Void {
		instances.push(inst);
	}

	/**
	* This function iterates over all registered instances and 
	* tests each for collision with the supplied Ball object.
	* Arguments:
	* obj - Ball object
	* Returns:
	* true if collision was detected; false otherwise.
	**/
	
	public static function testCollision(obj:Ball):Boolean {
		for (var w in instances) {
			if (instances[w].testColl(obj)) {
				return true;
			}
		}
			return false;
	}

	
	
	/**
	* Initializes object
	**/
	public function SolidBlock() {
		super();
		SolidBlock.addInstance(this);
	}




	public function testColl(targ:Ball):Boolean {
		if (!hitTest(targ.getTestBall())){
			return false;
		}
		//trace ("solid block hit!");
		var randSpeed = Math.random(2)-1;
		var bounds:Object = this.getBounds(CastleGame.getRoot());
		var xDir = 0;
		var yDir = 0;
		
		if (targ._x <= bounds.xMin) { xDir = -1; }
		if (targ._x >= bounds.xMax) { xDir = 1; }
		if (targ._y <= bounds.yMin) { yDir = -1; }
		if (targ._y >= bounds.yMax) { yDir = 1; } 
		
		if (yDir == 1){ 
			targ.reflectDown(randSpeed);
			}
		if (yDir == -1){ 
			targ.reflectUp(randSpeed);
			}
		if (xDir == 1){ 
			targ.reflectRight(randSpeed);
			}
		if (xDir == -1){ 
			targ.reflectLeft(randSpeed);
			}
		return true;

	}
}
